'------------------------------------------------------------------------------
' TITLE:    viewport
' DATE:     April 15, 1991
' AUTHOR: R. Gonzalez
'
' DESCRIPTION:  Demonstrates multiple viewports and world coordinates, as well
'        as matrix transformation.
'
' COMPILING:    Remove STATIC declarations, uncomment indicated lines
'     Check: Include MBPCs & MBLCs, Include runtime code, Make all arrays static,
'     Use default window & menu, (if available: Generate 68020 & 68881 code).
'
' (MODIFICATION HISTORY)
' DATE:
' AUTHOR:
' DESCRIPTION:
'------------------------------------------------------------------------------

' Since mac windows use local coords (origin at top left), simplified formulas
' for view transformation are used.  Two sets of formulas are used, one for
' the "Detail" viewport and one for the "Overall" viewport.
DEF FNdxv%(xw) = (xw-dxw1)*(dxv2%-dxv1%)/(dxw2-dxw1)
DEF FNdyv%(yw) = (yw-dyw1)*(dyv2%-dyv1%)/(dyw2-dyw1)
DEF FNoxv%(xw) = (xw-oxw1)*(oxv2%-oxv1%)/(oxw2-oxw1)
DEF FNoyv%(yw) = (yw-oyw1)*(oyv2%-oyv1%)/(oyw2-oyw1)

DIM SHARED dxw1,dxw2,dyw1,dyw2,dxv1%,dxv2%,dyv1%,dyv2%
DIM SHARED oxw1,oxw2,oyw1,oyw2,oxv1%,oxv2%,oyv1%,oyv2%
DIM SHARED pic%(2400),TRUE%,FALSE%,pi
DIM SHARED num.lines%,num.circles%,x(1000),y(1000),r(1000),trans(3,3),p(3),pprime(3)

'MAIN

    DIM menu.item%,closer%

    TRUE% = -1
    FALSE% = 0
    pi = 3.14159
    closer% = FALSE%
    
    WINDOW CLOSE 1  'close default window
    update.menu closer%
    set.up.wwindows closer%
    set.up.viewports
    initialize.objects
    draw.overall.view
    draw.detail.view

    'event loop (File and Edit menus handled automatically by QB)
    WHILE TRUE%
        IF MENU(0) = 3 THEN
            menu.item% = MENU(1)
            handle.options.menu menu.item%,closer%
        END IF
    WEND

END

'------------------------------------------------------------------------------
' draw menu bar
'------------------------------------------------------------------------------
SUB update.menu (closer%) STATIC

    'use default File and Edit menus

    MENU 3,0,1,"Options"

    IF closer% THEN
        MENU 3,1,1,"Reduce Detail"
    ELSE
        MENU 3,1,1,"Enlarge Detail"
    END IF

    MENU 3,2,1,"Rotate..."

END SUB

'------------------------------------------------------------------------------
' handle selection from options menu
'------------------------------------------------------------------------------
SUB handle.options.menu (item.no%,closer%) STATIC

    ' for compiler only:
'    dim rate%,success%

    SELECT CASE item.no%
        CASE 1:    IF closer% THEN
                             closer% = FALSE%
                         ELSE
                             closer% = TRUE%
                         END IF
                         set.up.wwindows closer%
                         draw.detail.view
        
        CASE 2:    get.rotate.rate rate,success%
                        IF success% THEN
                            rotate.object rate
                            draw.overall.view
                            draw.detail.view
                        END IF
        
        CASE ELSE:
        
    END SELECT

    update.menu closer%
    
END SUB

'------------------------------------------------------------------------------
' dialog box to get rotate rate
'------------------------------------------------------------------------------
SUB get.rotate.rate (rate,success%) STATIC

    ' for compiler only:
'    dim rect%(4),done%,event.type%

    'make window 1 the default, so you GET from it:
    WINDOW OUTPUT 1
    
    ' you need about 1 integer to store a 13 (?) square pixel area;
    ' make sure pic% is big enough!
    GET (42-dxv1%,42-dyv1%)-(237-dxv1%,207-dyv1%),pic%
    WINDOW 3,,(50,50)-(230,200),-2
    
    TEXTFONT 0
    setrect rect%(0),0,0,180,40
    textbox "Rotation amount in degrees:",rect%(0),0
    EDIT FIELD 1,"",(5,50)-(175,66),1
    fat.button 1,1,"OK",46,86
    BUTTON 2,1,"Cancel",(50,120)-(120,135),1

    ' local event loop since this is a modal alert
    done% = FALSE%
    WHILE NOT done%
        event.type% = DIALOG(0)
        IF event.type% = 1 THEN        'button pressed
            done% = TRUE%
            IF DIALOG(1) = 1 THEN
                rate = VAL(EDIT$(1))
                success% = TRUE%
            ELSE
                success% = FALSE%
            END IF
        ELSEIF event.type% = 6 THEN        'RETURN pressed
            done% = TRUE%
            rate = VAL(EDIT$(1))
            success% = TRUE%
        END IF
    WEND

    WINDOW CLOSE 3
    ' refresh the screen
    PUT (42-dxv1%,42-dyv1%)-(237-dxv1%,207-dyv1%),pic%,PSET

END SUB

'------------------------------------------------------------------------------
' draw button with double border
'------------------------------------------------------------------------------
SUB fat.button (button.no%,state%,message$,x%,y%) STATIC

    ' for compiler only:
'    dim rect%(4)

    setrect rect%(0),x%,y%,x%+76,y%+28
    PENSIZE 3,3
    FRAMEROUNDRECT VARPTR(rect%(0)),16,16
    PENNORMAL
    BUTTON button.no%,state%,message$,(x%+4,y%+4)-(x%+72,y%+24),1

END SUB
    
'------------------------------------------------------------------------------
' rotate object at rotate rate
'------------------------------------------------------------------------------
SUB rotate.object (rate) STATIC

    'for compiler only:
'    dim i%

    make.rotation.matrix rate*pi/180,trans()
    
    FOR i% = 1 TO 2*num.lines%+num.circles%
        p(1) = x(i%)
        p(2) = y(i%)
        p(3) = 1
        vect.mat p(),trans(),pprime()
        x(i%) = pprime(1)
        y(i%) = pprime(2)
    NEXT
    
END SUB

'------------------------------------------------------------------------------
' draw viewport with detail view
'------------------------------------------------------------------------------
SUB draw.detail.view STATIC

    'for compiler only:
'    dim i%

    WINDOW OUTPUT 1
    CLS

    FOR i% = 1 TO num.lines%
        LINE (FNdxv%(x(2*i%-1)),FNdyv%(y(2*i%-1)))-(FNdxv%(x(2*i%)),FNdyv%(y(2*i%)))
    NEXT

    FOR i% = 2*num.lines%+1 TO 2*num.lines%+num.circles%
        CIRCLE (FNdxv%(x(i%)),FNdyv%(y(i%))),r(i%)*(dxv2%-dxv1%)/(dxw2-dxw1)
    NEXT i%

END SUB

'------------------------------------------------------------------------------
' draw viewport with overview
'------------------------------------------------------------------------------
SUB draw.overall.view STATIC

    'for compiler only:
'    dim i%

    WINDOW OUTPUT 2
    CLS
    
    FOR i% = 1 TO num.lines%
        LINE (FNoxv%(x(2*i%-1)),FNoyv%(y(2*i%-1)))-(FNoxv%(x(2*i%)),FNoyv%(y(2*i%)))
    NEXT
    
    FOR i% = 2*num.lines%+1 TO 2*num.lines%+num.circles%
        CIRCLE (FNoxv%(x(i%)),FNoyv%(y(i%))),r(i%)*(oxv2%-oxv1%)/(oxw2-oxw1)
    NEXT i%
    
END SUB

'------------------------------------------------------------------------------
' setup viewports
'------------------------------------------------------------------------------
SUB set.up.viewports STATIC

    dxv1% = 30
    dyv1% = 30
    dxv2% = 330
    dyv2% = 330
    oxv1% = 350
    oyv1% = 100
    oxv2% = 450
    oyv2% = 200

    WINDOW 1,,(dxv1%,dyv1%)-(dxv2%,dyv2%),3
    WINDOW 2,,(oxv1%,oyv1%)-(oxv2%,oyv2%),3

END SUB

'------------------------------------------------------------------------------
' setup world coordinate system windows.  I call them "wwindows" to distinguish
' from the usual Macintosh meaning of an on-screen "window" (which I term "viewport"!)
'------------------------------------------------------------------------------
SUB set.up.wwindows (closer%) STATIC

    oxw1 = -2
    oxw2 = 2
    oyw1 = 2
    oyw2 = -2
    
    IF closer% THEN
        dxw1 = -.25
        dxw2 = .25
        dyw1 = .25
        dyw2 = -.25
    ELSE
        dxw1 = -.5
        dxw2 = .5
        dyw1 = .5
        dyw2 = -.5
    END IF

END SUB

'------------------------------------------------------------------------------
' initialize global object data
'------------------------------------------------------------------------------
SUB initialize.objects STATIC
    
    num.lines% = 5
    num.circles% = 1
    
    x(1) = 0    'torso
    y(1) = 0
    x(2) = 0
    y(2) = -.2
    
    x(3) = -.2    'left leg
    y(3) = -.4
    x(4) = 0
    y(4) = -.2

    x(5) = .2    'right leg
    y(5) = -.4
    x(6) = 0
    y(6) = -.2

    x(7) = -.3    'left arm
    y(7) = .1
    x(8) = 0
    y(8) = -.1

    x(9) = .3    'right arm
    y(9) = .1
    x(10) = 0
    y(10) = -.1

    x(11) = 0    'head
    y(11) = .1
    r(11) = .1    'radius of head
    
END SUB

'------------------------------------------------------------------------------
' multiply two 3x3 matrices - produces a 3x3 matrix
'------------------------------------------------------------------------------
SUB mat.mat (a(),b(),mresult()) STATIC

    'for compiler only:
'    dim r%,c%

    FOR r% = 1 TO 3
        FOR c% = 1 TO 3
            mresult(r%,c%) = a(r%,1)*b(1,c%)+a(r%,2)*b(2,c%)+a(r%,3)*b(3,c%)
        NEXT
    NEXT
    
END SUB

'------------------------------------------------------------------------------
' multiply 3-vector by 3x3 matrix - produces a 3-vector
'------------------------------------------------------------------------------
SUB vect.mat (v(),m(),vresult()) STATIC

    'for compiler only:
'    dim c%

    FOR c% = 1 TO 3
        vresult(c%) = v(1)*m(1,c%)+v(2)*m(2,c%)+v(3)*m(3,c%)
    NEXT
    
END SUB

'------------------------------------------------------------------------------
' create 3x3 rotation matrix
'------------------------------------------------------------------------------
SUB make.rotation.matrix (theta,m()) STATIC

    'for compiler only:
'    dim c%

    m(1,1) = COS(theta)
    m(1,2) = SIN(theta)
    m(1,3) = 0
    m(2,1) = -SIN(theta)
    m(2,2) = COS(theta)
    m(2,3) = 0
    m(3,1) = 0
    m(3,2) = 0
    m(3,3) = 1

END SUB

